# -*- coding: utf-8 -*-

"""
AWS CloudFormation resource specification is a machine-readable json file
that includes detailed information about AWS Resource, Property specification.

The AWS Resource class declaration code is auto generated base on this file.
It can be found at
https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cfn-resource-specification.html

This module aim to auto generate class declaration code from spec file using
jinja2 template engine.
"""
