# -*- coding: utf-8 -*-

class _ServicePrincipalMixin:
    @classmethod
    def accessanalyzer(cls): return cls._build("accessanalyzer.amazonaws.com") # pragma: no cover

    @classmethod
    def account(cls): return cls._build("account.amazonaws.com") # pragma: no cover

    @classmethod
    def acm_pca(cls): return cls._build("acm-pca.amazonaws.com") # pragma: no cover

    @classmethod
    def acm(cls): return cls._build("acm.amazonaws.com") # pragma: no cover

    @classmethod
    def alexaforbusiness(cls): return cls._build("alexaforbusiness.amazonaws.com") # pragma: no cover

    @classmethod
    def amp(cls): return cls._build("amp.amazonaws.com") # pragma: no cover

    @classmethod
    def amplify(cls): return cls._build("amplify.amazonaws.com") # pragma: no cover

    @classmethod
    def amplifybackend(cls): return cls._build("amplifybackend.amazonaws.com") # pragma: no cover

    @classmethod
    def amplifyuibuilder(cls): return cls._build("amplifyuibuilder.amazonaws.com") # pragma: no cover

    @classmethod
    def apigateway(cls): return cls._build("apigateway.amazonaws.com") # pragma: no cover

    @classmethod
    def apigatewaymanagementapi(cls): return cls._build("apigatewaymanagementapi.amazonaws.com") # pragma: no cover

    @classmethod
    def apigatewayv2(cls): return cls._build("apigatewayv2.amazonaws.com") # pragma: no cover

    @classmethod
    def appconfig(cls): return cls._build("appconfig.amazonaws.com") # pragma: no cover

    @classmethod
    def appconfigdata(cls): return cls._build("appconfigdata.amazonaws.com") # pragma: no cover

    @classmethod
    def appflow(cls): return cls._build("appflow.amazonaws.com") # pragma: no cover

    @classmethod
    def appintegrations(cls): return cls._build("appintegrations.amazonaws.com") # pragma: no cover

    @classmethod
    def application_autoscaling(cls): return cls._build("application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def application_insights(cls): return cls._build("application-insights.amazonaws.com") # pragma: no cover

    @classmethod
    def applicationcostprofiler(cls): return cls._build("applicationcostprofiler.amazonaws.com") # pragma: no cover

    @classmethod
    def appmesh(cls): return cls._build("appmesh.amazonaws.com") # pragma: no cover

    @classmethod
    def apprunner(cls): return cls._build("apprunner.amazonaws.com") # pragma: no cover

    @classmethod
    def appstream(cls): return cls._build("appstream.amazonaws.com") # pragma: no cover

    @classmethod
    def appsync(cls): return cls._build("appsync.amazonaws.com") # pragma: no cover

    @classmethod
    def athena(cls): return cls._build("athena.amazonaws.com") # pragma: no cover

    @classmethod
    def auditmanager(cls): return cls._build("auditmanager.amazonaws.com") # pragma: no cover

    @classmethod
    def autoscaling_plans(cls): return cls._build("autoscaling-plans.amazonaws.com") # pragma: no cover

    @classmethod
    def autoscaling(cls): return cls._build("autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def backup_gateway(cls): return cls._build("backup-gateway.amazonaws.com") # pragma: no cover

    @classmethod
    def backup(cls): return cls._build("backup.amazonaws.com") # pragma: no cover

    @classmethod
    def batch(cls): return cls._build("batch.amazonaws.com") # pragma: no cover

    @classmethod
    def billingconductor(cls): return cls._build("billingconductor.amazonaws.com") # pragma: no cover

    @classmethod
    def braket(cls): return cls._build("braket.amazonaws.com") # pragma: no cover

    @classmethod
    def budgets(cls): return cls._build("budgets.amazonaws.com") # pragma: no cover

    @classmethod
    def ce(cls): return cls._build("ce.amazonaws.com") # pragma: no cover

    @classmethod
    def chime_sdk_identity(cls): return cls._build("chime-sdk-identity.amazonaws.com") # pragma: no cover

    @classmethod
    def chime_sdk_meetings(cls): return cls._build("chime-sdk-meetings.amazonaws.com") # pragma: no cover

    @classmethod
    def chime_sdk_messaging(cls): return cls._build("chime-sdk-messaging.amazonaws.com") # pragma: no cover

    @classmethod
    def chime(cls): return cls._build("chime.amazonaws.com") # pragma: no cover

    @classmethod
    def cloud9(cls): return cls._build("cloud9.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudcontrol(cls): return cls._build("cloudcontrol.amazonaws.com") # pragma: no cover

    @classmethod
    def clouddirectory(cls): return cls._build("clouddirectory.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudformation(cls): return cls._build("cloudformation.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudfront(cls): return cls._build("cloudfront.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudhsm(cls): return cls._build("cloudhsm.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudhsmv2(cls): return cls._build("cloudhsmv2.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudsearch(cls): return cls._build("cloudsearch.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudsearchdomain(cls): return cls._build("cloudsearchdomain.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudtrail(cls): return cls._build("cloudtrail.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudwatch(cls): return cls._build("cloudwatch.amazonaws.com") # pragma: no cover

    @classmethod
    def codeartifact(cls): return cls._build("codeartifact.amazonaws.com") # pragma: no cover

    @classmethod
    def codebuild(cls): return cls._build("codebuild.amazonaws.com") # pragma: no cover

    @classmethod
    def codecommit(cls): return cls._build("codecommit.amazonaws.com") # pragma: no cover

    @classmethod
    def codedeploy(cls): return cls._build("codedeploy.amazonaws.com") # pragma: no cover

    @classmethod
    def codeguru_reviewer(cls): return cls._build("codeguru-reviewer.amazonaws.com") # pragma: no cover

    @classmethod
    def codeguruprofiler(cls): return cls._build("codeguruprofiler.amazonaws.com") # pragma: no cover

    @classmethod
    def codepipeline(cls): return cls._build("codepipeline.amazonaws.com") # pragma: no cover

    @classmethod
    def codestar_connections(cls): return cls._build("codestar-connections.amazonaws.com") # pragma: no cover

    @classmethod
    def codestar_notifications(cls): return cls._build("codestar-notifications.amazonaws.com") # pragma: no cover

    @classmethod
    def codestar(cls): return cls._build("codestar.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_identity(cls): return cls._build("cognito-identity.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_idp(cls): return cls._build("cognito-idp.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_sync(cls): return cls._build("cognito-sync.amazonaws.com") # pragma: no cover

    @classmethod
    def comprehend(cls): return cls._build("comprehend.amazonaws.com") # pragma: no cover

    @classmethod
    def comprehendmedical(cls): return cls._build("comprehendmedical.amazonaws.com") # pragma: no cover

    @classmethod
    def compute_optimizer(cls): return cls._build("compute-optimizer.amazonaws.com") # pragma: no cover

    @classmethod
    def config(cls): return cls._build("config.amazonaws.com") # pragma: no cover

    @classmethod
    def connect_contact_lens(cls): return cls._build("connect-contact-lens.amazonaws.com") # pragma: no cover

    @classmethod
    def connect(cls): return cls._build("connect.amazonaws.com") # pragma: no cover

    @classmethod
    def connectparticipant(cls): return cls._build("connectparticipant.amazonaws.com") # pragma: no cover

    @classmethod
    def cur(cls): return cls._build("cur.amazonaws.com") # pragma: no cover

    @classmethod
    def customer_profiles(cls): return cls._build("customer-profiles.amazonaws.com") # pragma: no cover

    @classmethod
    def databrew(cls): return cls._build("databrew.amazonaws.com") # pragma: no cover

    @classmethod
    def dataexchange(cls): return cls._build("dataexchange.amazonaws.com") # pragma: no cover

    @classmethod
    def datapipeline(cls): return cls._build("datapipeline.amazonaws.com") # pragma: no cover

    @classmethod
    def datasync(cls): return cls._build("datasync.amazonaws.com") # pragma: no cover

    @classmethod
    def dax(cls): return cls._build("dax.amazonaws.com") # pragma: no cover

    @classmethod
    def detective(cls): return cls._build("detective.amazonaws.com") # pragma: no cover

    @classmethod
    def devicefarm(cls): return cls._build("devicefarm.amazonaws.com") # pragma: no cover

    @classmethod
    def devops_guru(cls): return cls._build("devops-guru.amazonaws.com") # pragma: no cover

    @classmethod
    def directconnect(cls): return cls._build("directconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def discovery(cls): return cls._build("discovery.amazonaws.com") # pragma: no cover

    @classmethod
    def dlm(cls): return cls._build("dlm.amazonaws.com") # pragma: no cover

    @classmethod
    def dms(cls): return cls._build("dms.amazonaws.com") # pragma: no cover

    @classmethod
    def docdb(cls): return cls._build("docdb.amazonaws.com") # pragma: no cover

    @classmethod
    def drs(cls): return cls._build("drs.amazonaws.com") # pragma: no cover

    @classmethod
    def ds(cls): return cls._build("ds.amazonaws.com") # pragma: no cover

    @classmethod
    def dynamodb(cls): return cls._build("dynamodb.amazonaws.com") # pragma: no cover

    @classmethod
    def dynamodbstreams(cls): return cls._build("dynamodbstreams.amazonaws.com") # pragma: no cover

    @classmethod
    def ebs(cls): return cls._build("ebs.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2_instance_connect(cls): return cls._build("ec2-instance-connect.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2(cls): return cls._build("ec2.amazonaws.com") # pragma: no cover

    @classmethod
    def ecr_public(cls): return cls._build("ecr-public.amazonaws.com") # pragma: no cover

    @classmethod
    def ecr(cls): return cls._build("ecr.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs(cls): return cls._build("ecs.amazonaws.com") # pragma: no cover

    @classmethod
    def efs(cls): return cls._build("efs.amazonaws.com") # pragma: no cover

    @classmethod
    def eks(cls): return cls._build("eks.amazonaws.com") # pragma: no cover

    @classmethod
    def elastic_inference(cls): return cls._build("elastic-inference.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticache(cls): return cls._build("elasticache.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticbeanstalk(cls): return cls._build("elasticbeanstalk.amazonaws.com") # pragma: no cover

    @classmethod
    def elastictranscoder(cls): return cls._build("elastictranscoder.amazonaws.com") # pragma: no cover

    @classmethod
    def elb(cls): return cls._build("elb.amazonaws.com") # pragma: no cover

    @classmethod
    def elbv2(cls): return cls._build("elbv2.amazonaws.com") # pragma: no cover

    @classmethod
    def emr_containers(cls): return cls._build("emr-containers.amazonaws.com") # pragma: no cover

    @classmethod
    def emr(cls): return cls._build("emr.amazonaws.com") # pragma: no cover

    @classmethod
    def es(cls): return cls._build("es.amazonaws.com") # pragma: no cover

    @classmethod
    def events(cls): return cls._build("events.amazonaws.com") # pragma: no cover

    @classmethod
    def evidently(cls): return cls._build("evidently.amazonaws.com") # pragma: no cover

    @classmethod
    def finspace_data(cls): return cls._build("finspace-data.amazonaws.com") # pragma: no cover

    @classmethod
    def finspace(cls): return cls._build("finspace.amazonaws.com") # pragma: no cover

    @classmethod
    def firehose(cls): return cls._build("firehose.amazonaws.com") # pragma: no cover

    @classmethod
    def fis(cls): return cls._build("fis.amazonaws.com") # pragma: no cover

    @classmethod
    def fms(cls): return cls._build("fms.amazonaws.com") # pragma: no cover

    @classmethod
    def forecast(cls): return cls._build("forecast.amazonaws.com") # pragma: no cover

    @classmethod
    def forecastquery(cls): return cls._build("forecastquery.amazonaws.com") # pragma: no cover

    @classmethod
    def frauddetector(cls): return cls._build("frauddetector.amazonaws.com") # pragma: no cover

    @classmethod
    def fsx(cls): return cls._build("fsx.amazonaws.com") # pragma: no cover

    @classmethod
    def gamelift(cls): return cls._build("gamelift.amazonaws.com") # pragma: no cover

    @classmethod
    def gamesparks(cls): return cls._build("gamesparks.amazonaws.com") # pragma: no cover

    @classmethod
    def glacier(cls): return cls._build("glacier.amazonaws.com") # pragma: no cover

    @classmethod
    def globalaccelerator(cls): return cls._build("globalaccelerator.amazonaws.com") # pragma: no cover

    @classmethod
    def glue(cls): return cls._build("glue.amazonaws.com") # pragma: no cover

    @classmethod
    def grafana(cls): return cls._build("grafana.amazonaws.com") # pragma: no cover

    @classmethod
    def greengrass(cls): return cls._build("greengrass.amazonaws.com") # pragma: no cover

    @classmethod
    def greengrassv2(cls): return cls._build("greengrassv2.amazonaws.com") # pragma: no cover

    @classmethod
    def groundstation(cls): return cls._build("groundstation.amazonaws.com") # pragma: no cover

    @classmethod
    def guardduty(cls): return cls._build("guardduty.amazonaws.com") # pragma: no cover

    @classmethod
    def health(cls): return cls._build("health.amazonaws.com") # pragma: no cover

    @classmethod
    def healthlake(cls): return cls._build("healthlake.amazonaws.com") # pragma: no cover

    @classmethod
    def honeycode(cls): return cls._build("honeycode.amazonaws.com") # pragma: no cover

    @classmethod
    def iam(cls): return cls._build("iam.amazonaws.com") # pragma: no cover

    @classmethod
    def identitystore(cls): return cls._build("identitystore.amazonaws.com") # pragma: no cover

    @classmethod
    def imagebuilder(cls): return cls._build("imagebuilder.amazonaws.com") # pragma: no cover

    @classmethod
    def importexport(cls): return cls._build("importexport.amazonaws.com") # pragma: no cover

    @classmethod
    def inspector(cls): return cls._build("inspector.amazonaws.com") # pragma: no cover

    @classmethod
    def inspector2(cls): return cls._build("inspector2.amazonaws.com") # pragma: no cover

    @classmethod
    def iot_data(cls): return cls._build("iot-data.amazonaws.com") # pragma: no cover

    @classmethod
    def iot_jobs_data(cls): return cls._build("iot-jobs-data.amazonaws.com") # pragma: no cover

    @classmethod
    def iot(cls): return cls._build("iot.amazonaws.com") # pragma: no cover

    @classmethod
    def iot1click_devices(cls): return cls._build("iot1click-devices.amazonaws.com") # pragma: no cover

    @classmethod
    def iot1click_projects(cls): return cls._build("iot1click-projects.amazonaws.com") # pragma: no cover

    @classmethod
    def iotanalytics(cls): return cls._build("iotanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def iotdeviceadvisor(cls): return cls._build("iotdeviceadvisor.amazonaws.com") # pragma: no cover

    @classmethod
    def iotevents_data(cls): return cls._build("iotevents-data.amazonaws.com") # pragma: no cover

    @classmethod
    def iotevents(cls): return cls._build("iotevents.amazonaws.com") # pragma: no cover

    @classmethod
    def iotfleethub(cls): return cls._build("iotfleethub.amazonaws.com") # pragma: no cover

    @classmethod
    def iotsecuretunneling(cls): return cls._build("iotsecuretunneling.amazonaws.com") # pragma: no cover

    @classmethod
    def iotsitewise(cls): return cls._build("iotsitewise.amazonaws.com") # pragma: no cover

    @classmethod
    def iotthingsgraph(cls): return cls._build("iotthingsgraph.amazonaws.com") # pragma: no cover

    @classmethod
    def iottwinmaker(cls): return cls._build("iottwinmaker.amazonaws.com") # pragma: no cover

    @classmethod
    def iotwireless(cls): return cls._build("iotwireless.amazonaws.com") # pragma: no cover

    @classmethod
    def ivs(cls): return cls._build("ivs.amazonaws.com") # pragma: no cover

    @classmethod
    def kafka(cls): return cls._build("kafka.amazonaws.com") # pragma: no cover

    @classmethod
    def kafkaconnect(cls): return cls._build("kafkaconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def kendra(cls): return cls._build("kendra.amazonaws.com") # pragma: no cover

    @classmethod
    def keyspaces(cls): return cls._build("keyspaces.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesis_video_archived_media(cls): return cls._build("kinesis-video-archived-media.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesis_video_media(cls): return cls._build("kinesis-video-media.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesis_video_signaling(cls): return cls._build("kinesis-video-signaling.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesis(cls): return cls._build("kinesis.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesisanalytics(cls): return cls._build("kinesisanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesisanalyticsv2(cls): return cls._build("kinesisanalyticsv2.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesisvideo(cls): return cls._build("kinesisvideo.amazonaws.com") # pragma: no cover

    @classmethod
    def kms(cls): return cls._build("kms.amazonaws.com") # pragma: no cover

    @classmethod
    def lakeformation(cls): return cls._build("lakeformation.amazonaws.com") # pragma: no cover

    @classmethod
    def awslambda(cls): return cls._build("lambda.amazonaws.com") # pragma: no cover

    @classmethod
    def lex_models(cls): return cls._build("lex-models.amazonaws.com") # pragma: no cover

    @classmethod
    def lex_runtime(cls): return cls._build("lex-runtime.amazonaws.com") # pragma: no cover

    @classmethod
    def lexv2_models(cls): return cls._build("lexv2-models.amazonaws.com") # pragma: no cover

    @classmethod
    def lexv2_runtime(cls): return cls._build("lexv2-runtime.amazonaws.com") # pragma: no cover

    @classmethod
    def license_manager(cls): return cls._build("license-manager.amazonaws.com") # pragma: no cover

    @classmethod
    def lightsail(cls): return cls._build("lightsail.amazonaws.com") # pragma: no cover

    @classmethod
    def location(cls): return cls._build("location.amazonaws.com") # pragma: no cover

    @classmethod
    def logs(cls): return cls._build("logs.amazonaws.com") # pragma: no cover

    @classmethod
    def lookoutequipment(cls): return cls._build("lookoutequipment.amazonaws.com") # pragma: no cover

    @classmethod
    def lookoutmetrics(cls): return cls._build("lookoutmetrics.amazonaws.com") # pragma: no cover

    @classmethod
    def lookoutvision(cls): return cls._build("lookoutvision.amazonaws.com") # pragma: no cover

    @classmethod
    def machinelearning(cls): return cls._build("machinelearning.amazonaws.com") # pragma: no cover

    @classmethod
    def macie(cls): return cls._build("macie.amazonaws.com") # pragma: no cover

    @classmethod
    def macie2(cls): return cls._build("macie2.amazonaws.com") # pragma: no cover

    @classmethod
    def managedblockchain(cls): return cls._build("managedblockchain.amazonaws.com") # pragma: no cover

    @classmethod
    def marketplace_catalog(cls): return cls._build("marketplace-catalog.amazonaws.com") # pragma: no cover

    @classmethod
    def marketplace_entitlement(cls): return cls._build("marketplace-entitlement.amazonaws.com") # pragma: no cover

    @classmethod
    def marketplacecommerceanalytics(cls): return cls._build("marketplacecommerceanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def mediaconnect(cls): return cls._build("mediaconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def mediaconvert(cls): return cls._build("mediaconvert.amazonaws.com") # pragma: no cover

    @classmethod
    def medialive(cls): return cls._build("medialive.amazonaws.com") # pragma: no cover

    @classmethod
    def mediapackage_vod(cls): return cls._build("mediapackage-vod.amazonaws.com") # pragma: no cover

    @classmethod
    def mediapackage(cls): return cls._build("mediapackage.amazonaws.com") # pragma: no cover

    @classmethod
    def mediastore_data(cls): return cls._build("mediastore-data.amazonaws.com") # pragma: no cover

    @classmethod
    def mediastore(cls): return cls._build("mediastore.amazonaws.com") # pragma: no cover

    @classmethod
    def mediatailor(cls): return cls._build("mediatailor.amazonaws.com") # pragma: no cover

    @classmethod
    def memorydb(cls): return cls._build("memorydb.amazonaws.com") # pragma: no cover

    @classmethod
    def meteringmarketplace(cls): return cls._build("meteringmarketplace.amazonaws.com") # pragma: no cover

    @classmethod
    def mgh(cls): return cls._build("mgh.amazonaws.com") # pragma: no cover

    @classmethod
    def mgn(cls): return cls._build("mgn.amazonaws.com") # pragma: no cover

    @classmethod
    def migration_hub_refactor_spaces(cls): return cls._build("migration-hub-refactor-spaces.amazonaws.com") # pragma: no cover

    @classmethod
    def migrationhub_config(cls): return cls._build("migrationhub-config.amazonaws.com") # pragma: no cover

    @classmethod
    def migrationhubstrategy(cls): return cls._build("migrationhubstrategy.amazonaws.com") # pragma: no cover

    @classmethod
    def mobile(cls): return cls._build("mobile.amazonaws.com") # pragma: no cover

    @classmethod
    def mq(cls): return cls._build("mq.amazonaws.com") # pragma: no cover

    @classmethod
    def mturk(cls): return cls._build("mturk.amazonaws.com") # pragma: no cover

    @classmethod
    def mwaa(cls): return cls._build("mwaa.amazonaws.com") # pragma: no cover

    @classmethod
    def neptune(cls): return cls._build("neptune.amazonaws.com") # pragma: no cover

    @classmethod
    def network_firewall(cls): return cls._build("network-firewall.amazonaws.com") # pragma: no cover

    @classmethod
    def networkmanager(cls): return cls._build("networkmanager.amazonaws.com") # pragma: no cover

    @classmethod
    def nimble(cls): return cls._build("nimble.amazonaws.com") # pragma: no cover

    @classmethod
    def opensearch(cls): return cls._build("opensearch.amazonaws.com") # pragma: no cover

    @classmethod
    def opsworks(cls): return cls._build("opsworks.amazonaws.com") # pragma: no cover

    @classmethod
    def opsworkscm(cls): return cls._build("opsworkscm.amazonaws.com") # pragma: no cover

    @classmethod
    def organizations(cls): return cls._build("organizations.amazonaws.com") # pragma: no cover

    @classmethod
    def outposts(cls): return cls._build("outposts.amazonaws.com") # pragma: no cover

    @classmethod
    def panorama(cls): return cls._build("panorama.amazonaws.com") # pragma: no cover

    @classmethod
    def personalize_events(cls): return cls._build("personalize-events.amazonaws.com") # pragma: no cover

    @classmethod
    def personalize_runtime(cls): return cls._build("personalize-runtime.amazonaws.com") # pragma: no cover

    @classmethod
    def personalize(cls): return cls._build("personalize.amazonaws.com") # pragma: no cover

    @classmethod
    def pi(cls): return cls._build("pi.amazonaws.com") # pragma: no cover

    @classmethod
    def pinpoint_email(cls): return cls._build("pinpoint-email.amazonaws.com") # pragma: no cover

    @classmethod
    def pinpoint_sms_voice_v2(cls): return cls._build("pinpoint-sms-voice-v2.amazonaws.com") # pragma: no cover

    @classmethod
    def pinpoint_sms_voice(cls): return cls._build("pinpoint-sms-voice.amazonaws.com") # pragma: no cover

    @classmethod
    def pinpoint(cls): return cls._build("pinpoint.amazonaws.com") # pragma: no cover

    @classmethod
    def polly(cls): return cls._build("polly.amazonaws.com") # pragma: no cover

    @classmethod
    def pricing(cls): return cls._build("pricing.amazonaws.com") # pragma: no cover

    @classmethod
    def proton(cls): return cls._build("proton.amazonaws.com") # pragma: no cover

    @classmethod
    def qldb_session(cls): return cls._build("qldb-session.amazonaws.com") # pragma: no cover

    @classmethod
    def qldb(cls): return cls._build("qldb.amazonaws.com") # pragma: no cover

    @classmethod
    def quicksight(cls): return cls._build("quicksight.amazonaws.com") # pragma: no cover

    @classmethod
    def ram(cls): return cls._build("ram.amazonaws.com") # pragma: no cover

    @classmethod
    def rbin(cls): return cls._build("rbin.amazonaws.com") # pragma: no cover

    @classmethod
    def rds_data(cls): return cls._build("rds-data.amazonaws.com") # pragma: no cover

    @classmethod
    def rds(cls): return cls._build("rds.amazonaws.com") # pragma: no cover

    @classmethod
    def redshift_data(cls): return cls._build("redshift-data.amazonaws.com") # pragma: no cover

    @classmethod
    def redshift(cls): return cls._build("redshift.amazonaws.com") # pragma: no cover

    @classmethod
    def rekognition(cls): return cls._build("rekognition.amazonaws.com") # pragma: no cover

    @classmethod
    def resiliencehub(cls): return cls._build("resiliencehub.amazonaws.com") # pragma: no cover

    @classmethod
    def resource_groups(cls): return cls._build("resource-groups.amazonaws.com") # pragma: no cover

    @classmethod
    def resourcegroupstaggingapi(cls): return cls._build("resourcegroupstaggingapi.amazonaws.com") # pragma: no cover

    @classmethod
    def robomaker(cls): return cls._build("robomaker.amazonaws.com") # pragma: no cover

    @classmethod
    def route53_recovery_cluster(cls): return cls._build("route53-recovery-cluster.amazonaws.com") # pragma: no cover

    @classmethod
    def route53_recovery_control_config(cls): return cls._build("route53-recovery-control-config.amazonaws.com") # pragma: no cover

    @classmethod
    def route53_recovery_readiness(cls): return cls._build("route53-recovery-readiness.amazonaws.com") # pragma: no cover

    @classmethod
    def route53(cls): return cls._build("route53.amazonaws.com") # pragma: no cover

    @classmethod
    def route53domains(cls): return cls._build("route53domains.amazonaws.com") # pragma: no cover

    @classmethod
    def route53resolver(cls): return cls._build("route53resolver.amazonaws.com") # pragma: no cover

    @classmethod
    def rum(cls): return cls._build("rum.amazonaws.com") # pragma: no cover

    @classmethod
    def s3(cls): return cls._build("s3.amazonaws.com") # pragma: no cover

    @classmethod
    def s3control(cls): return cls._build("s3control.amazonaws.com") # pragma: no cover

    @classmethod
    def s3outposts(cls): return cls._build("s3outposts.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker_a2i_runtime(cls): return cls._build("sagemaker-a2i-runtime.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker_edge(cls): return cls._build("sagemaker-edge.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker_featurestore_runtime(cls): return cls._build("sagemaker-featurestore-runtime.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker_runtime(cls): return cls._build("sagemaker-runtime.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker(cls): return cls._build("sagemaker.amazonaws.com") # pragma: no cover

    @classmethod
    def savingsplans(cls): return cls._build("savingsplans.amazonaws.com") # pragma: no cover

    @classmethod
    def schemas(cls): return cls._build("schemas.amazonaws.com") # pragma: no cover

    @classmethod
    def sdb(cls): return cls._build("sdb.amazonaws.com") # pragma: no cover

    @classmethod
    def secretsmanager(cls): return cls._build("secretsmanager.amazonaws.com") # pragma: no cover

    @classmethod
    def securityhub(cls): return cls._build("securityhub.amazonaws.com") # pragma: no cover

    @classmethod
    def serverlessrepo(cls): return cls._build("serverlessrepo.amazonaws.com") # pragma: no cover

    @classmethod
    def service_quotas(cls): return cls._build("service-quotas.amazonaws.com") # pragma: no cover

    @classmethod
    def servicecatalog_appregistry(cls): return cls._build("servicecatalog-appregistry.amazonaws.com") # pragma: no cover

    @classmethod
    def servicecatalog(cls): return cls._build("servicecatalog.amazonaws.com") # pragma: no cover

    @classmethod
    def servicediscovery(cls): return cls._build("servicediscovery.amazonaws.com") # pragma: no cover

    @classmethod
    def ses(cls): return cls._build("ses.amazonaws.com") # pragma: no cover

    @classmethod
    def sesv2(cls): return cls._build("sesv2.amazonaws.com") # pragma: no cover

    @classmethod
    def shield(cls): return cls._build("shield.amazonaws.com") # pragma: no cover

    @classmethod
    def signer(cls): return cls._build("signer.amazonaws.com") # pragma: no cover

    @classmethod
    def sms_voice(cls): return cls._build("sms-voice.amazonaws.com") # pragma: no cover

    @classmethod
    def sms(cls): return cls._build("sms.amazonaws.com") # pragma: no cover

    @classmethod
    def snow_device_management(cls): return cls._build("snow-device-management.amazonaws.com") # pragma: no cover

    @classmethod
    def snowball(cls): return cls._build("snowball.amazonaws.com") # pragma: no cover

    @classmethod
    def sns(cls): return cls._build("sns.amazonaws.com") # pragma: no cover

    @classmethod
    def sqs(cls): return cls._build("sqs.amazonaws.com") # pragma: no cover

    @classmethod
    def ssm_contacts(cls): return cls._build("ssm-contacts.amazonaws.com") # pragma: no cover

    @classmethod
    def ssm_incidents(cls): return cls._build("ssm-incidents.amazonaws.com") # pragma: no cover

    @classmethod
    def ssm(cls): return cls._build("ssm.amazonaws.com") # pragma: no cover

    @classmethod
    def sso_admin(cls): return cls._build("sso-admin.amazonaws.com") # pragma: no cover

    @classmethod
    def sso_oidc(cls): return cls._build("sso-oidc.amazonaws.com") # pragma: no cover

    @classmethod
    def sso(cls): return cls._build("sso.amazonaws.com") # pragma: no cover

    @classmethod
    def stepfunctions(cls): return cls._build("states.amazonaws.com") # pragma: no cover

    @classmethod
    def storagegateway(cls): return cls._build("storagegateway.amazonaws.com") # pragma: no cover

    @classmethod
    def sts(cls): return cls._build("sts.amazonaws.com") # pragma: no cover

    @classmethod
    def support(cls): return cls._build("support.amazonaws.com") # pragma: no cover

    @classmethod
    def swf(cls): return cls._build("swf.amazonaws.com") # pragma: no cover

    @classmethod
    def synthetics(cls): return cls._build("synthetics.amazonaws.com") # pragma: no cover

    @classmethod
    def textract(cls): return cls._build("textract.amazonaws.com") # pragma: no cover

    @classmethod
    def timestream_query(cls): return cls._build("timestream-query.amazonaws.com") # pragma: no cover

    @classmethod
    def timestream_write(cls): return cls._build("timestream-write.amazonaws.com") # pragma: no cover

    @classmethod
    def transcribe(cls): return cls._build("transcribe.amazonaws.com") # pragma: no cover

    @classmethod
    def transfer(cls): return cls._build("transfer.amazonaws.com") # pragma: no cover

    @classmethod
    def translate(cls): return cls._build("translate.amazonaws.com") # pragma: no cover

    @classmethod
    def voice_id(cls): return cls._build("voice-id.amazonaws.com") # pragma: no cover

    @classmethod
    def waf_regional(cls): return cls._build("waf-regional.amazonaws.com") # pragma: no cover

    @classmethod
    def waf(cls): return cls._build("waf.amazonaws.com") # pragma: no cover

    @classmethod
    def wafv2(cls): return cls._build("wafv2.amazonaws.com") # pragma: no cover

    @classmethod
    def wellarchitected(cls): return cls._build("wellarchitected.amazonaws.com") # pragma: no cover

    @classmethod
    def wisdom(cls): return cls._build("wisdom.amazonaws.com") # pragma: no cover

    @classmethod
    def workdocs(cls): return cls._build("workdocs.amazonaws.com") # pragma: no cover

    @classmethod
    def worklink(cls): return cls._build("worklink.amazonaws.com") # pragma: no cover

    @classmethod
    def workmail(cls): return cls._build("workmail.amazonaws.com") # pragma: no cover

    @classmethod
    def workmailmessageflow(cls): return cls._build("workmailmessageflow.amazonaws.com") # pragma: no cover

    @classmethod
    def workspaces_web(cls): return cls._build("workspaces-web.amazonaws.com") # pragma: no cover

    @classmethod
    def workspaces(cls): return cls._build("workspaces.amazonaws.com") # pragma: no cover

    @classmethod
    def xray(cls): return cls._build("xray.amazonaws.com") # pragma: no cover
