# -*- coding: utf-8 -*-

from . import (
    lakeformation,
    elasticache,
    appflow,
    glue,
    elasticbeanstalk,
    sagemaker,
    apprunner,
    s3,
    dms,
    iotanalytics,
    cloudtrail,
    ses,
    autoscaling,
    iotevents,
    iot,
    emr,
    wafv2,
    opensearchservice,
    pipes,
    mediapackage,
    acmpca,
    fsx,
    datasync,
    ec2,
    iotwireless,
    kinesisanalyticsv2,
    cognito,
    kinesisanalytics,
    opsworks,
    codedeploy,
    quicksight,
    appmesh,
    batch,
    applicationautoscaling,
    imagebuilder,
    appsync,
    medialive,
    amplifyuibuilder,
    awslambda,
    greengrassv2,
    waf,
    networkfirewall,
    lightsail,
    apigateway,
    billingconductor,
    evidently,
    appconfig,
    lex,
    config,
    resourcegroups,
    ecr,
    scheduler,
    groundstation,
    stepfunctions,
    ecs,
    pinpointemail,
    databrew,
    backup,
    gamelift,
    dynamodb,
    guardduty,
    elasticsearch,
    cloudwatch,
    greengrass,
    redshift,
    iottwinmaker,
    elasticloadbalancingv2,
    customerprofiles,
    budgets,
    networkmanager,
    rds,
    wisdom,
    codestar,
    ssmcontacts,
    pinpoint,
    iotfleetwise,
    cassandra,
    events,
    cloudfront,
    kendra,
    s3outposts,
    ssm,
    connect,
    kafkaconnect,
    appstream,
    route53,
    cloudformation,
    route53resolver,
    inspectorv2,
    timestream,
    codepipeline,
    identitystore,
    nimblestudio,
    eventschemas,
    globalaccelerator,
    ssmincidents,
    amazonmq,
    m2,
    dlm,
    datapipeline,
    iotsitewise,
    codebuild,
    rekognition,
    applicationinsights,
    location,
    apigatewayv2,
    licensemanager,
    mediastore,
    signer,
    fms,
    amplify,
    kinesisfirehose,
    macie,
    efs,
    ivs,
    frauddetector,
    autoscalingplans,
    synthetics,
    managedblockchain,
    auditmanager,
    s3objectlambda,
    elasticloadbalancing,
    mediatailor,
    msk,
    rolesanywhere,
    lookoutmetrics,
    eks,
    qldb,
    healthlake,
    mediaconvert,
    refactorspaces,
    servicediscovery,
    ce,
    appintegrations,
    wafregional,
    emrserverless,
    iam,
    devopsguru,
    fis,
    athena,
    route53recoverycontrol,
    sso,
    panorama,
    robomaker,
    finspace,
    resiliencehub,
    transfer,
    mwaa,
    iot1click,
    connectcampaigns,
    rum,
    aps,
    emrcontainers,
    codeguruprofiler,
    kinesis,
    opsworkscm,
    personalize,
    codestarnotifications,
    xray,
    accessanalyzer,
    mediaconnect,
    directoryservice,
    route53recoveryreadiness,
    logs,
    secretsmanager,
    sns,
    redshiftserverless,
    servicecatalog,
    resourceexplorer2,
    ask,
    certificatemanager,
    iotthingsgraph,
    memorydb,
    codecommit,
    neptune,
    workspaces,
    voiceid,
    dax,
    cloud9,
    ram,
    iotfleethub,
    oam,
    docdb,
    kms,
    detective,
    codegurureviewer,
    iotcoredeviceadvisor,
    inspector,
    supportapp,
    controltower,
    kinesisvideo,
    codeartifact,
    sqs,
    lookoutvision,
    forecast,
    chatbot,
    lookoutequipment,
    codestarconnections,
    servicecatalogappregistry,
    organizations,
    sdb,
    securityhub,
    cur,
)
